/*****************************************************************/
/*  Label connected clusters using the get/send/scan algorithm.  */
/*                                                               */
/*        John Apostolakis and Paul Coddington, May 1992         */
/*                                                               */
/*****************************************************************/

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <cscomm.h>

#include "sizes.h"
#include "types.h"
#include "comlat.h"
#include "addecl.h"
#include "latprint.h"
#include "timenos.h"

bool:lattice  boundary_fnd[2] ;

AuxType:lattice old_label;

int  get_every=2,  get_first=5;
int send_every=2, send_first=5; 
int scan_every=1, scan_first=-1;

int find_cluster( bool:lattice  boundary_inp[2] )
{
  int iter_count;
  void local_iter(), scan_iter(), one_get(), one_send();

  /* Copy the connections into new arrays, so they can be improved */
  boundary_fnd[0] = boundary_inp[0] ;
  boundary_fnd[1] = boundary_inp[1] ;

  /* Initialize the cluster labels */
  Cluster_label = pcoord(0) + SIZE * pcoord(1); 

  iter_count=0;
  do {

     /* Update the labels */
     old_label=Cluster_label;

     /* Do a local iteration */
     local_iter();

     /* Send the current label to the root of the cluster */
     if( (iter_count>=send_first) && (send_every!=0) && 
         ((send_every==1) || (((iter_count-send_first)%send_every)==0)) ) {
        one_send();
     }

     /* Propagate the labels with a scan */
     if( (iter_count>=scan_first) && (scan_every!=0) &&
         ((scan_every==1) || (((iter_count-scan_first)%scan_every)==0)) ) {
        scan_iter();
     }

     /* Get the current label from the root of the cluster */
     if( (iter_count>=get_first) && (get_every!=0) && 
         ((get_every==1) || (((iter_count-get_first)%get_every)==0)) ) {
        one_get();
     }

     iter_count++;

  } while ( |= (Cluster_label != old_label) );
  /* loop until there is no processor whose label has changed */

  return iter_count;

} /*******   end of find_cluster routine *******/

